// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package config

type LogConfig struct {
	Path       string
	Level      string
	MaxSize    int
	MaxBackups int
	MaxAge     int
	Stacktrace bool
}

type MysqlConfig struct {
	User     string
	Password string
	Addr     string
	DB       string
	Charset  string
	MaxIdle  int
	MaxConn  int
	LogMode  bool
}

type RedisConfig struct {
	Addrs        []string
	Password     string
	DB           int
	PoolSize     int
	MinIdleConns int
	MaxRetries   int
	Cluster      bool
}

type ElasticConfig struct {
	Addrs []string
	Sniff bool
}

type ThirdPartNode struct {
	Host      string
	AppKey    string
	AppSecret string
}

type ThirdPartConfig struct {
	PartName ThirdPartNode
}

type RPCNode struct {
	ServiceName   string
	ServicePort   int
	ServiceIp     string
	MysqlDb       string
	RedisDb       int
	LogLevel      string
	LogStacktrace bool
}

type KeepaliveConfig struct {
	ClientTime    int
	ClientTimeout int

	ServerTime     int
	ServerTimeout  int
	ServerMiniTime int
}

type RPCConfig struct {
	Keepalive  KeepaliveConfig
	SmartAuth  RPCNode
	SmartAlarm RPCNode
	//DustMonitor RPCNode
}

type OssConfig struct {
	Endpoint string
	Id       string
	Key      string
	Bucket   string
}

type InfluxdbConfig struct {
	Addr     string
	Username string
	Password string
}

type MqttConfig struct {
	Addr     string
	Username string
	Password string
	Topic string
	ResTopicPrefix string
}

type GatewayConfig struct {
	ServiceName   string
	ServiceIp     string
	ServicePort   int
	HeartPort int
	MysqlDb       string
	RedisDb       int
	LogLevel      string
	LogStacktrace bool
}

type Configure struct {
	// 基础配置
	//LocalMode bool
	K8s     bool
	RunMode string
	Log     LogConfig

	// 按需配置
	Influxdb InfluxdbConfig
	Mqtt MqttConfig
	// 所要启用的服务
	Rpc         RPCConfig
	TowerMonitor GatewayConfig
}
