// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package util

import (
	"context"
	"fmt"
	"tower-monitor/consts"
	"tower-monitor/pb"
	pb_v1 "tower-monitor/pb/v1"

	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)

var snKey map[string]*pb_v1.DeviceInfoReply

// 初始化map
func Init() {
	snKey = make(map[string]*pb_v1.DeviceInfoReply)
}

// 获取key
func GetKey(sn string) (*pb_v1.DeviceInfoReply, error) {
	if value, ok := snKey[sn]; ok {
		return value, nil
	}
	reply, err := GetDeviceInfo(sn)
	if err == nil {
		snKey[sn] = reply
	}
	return reply, err
}

// 从数据库中获取key
func GetDeviceInfo(sn string) (*pb_v1.DeviceInfoReply, error) {
	req := &pb_v1.DeviceInfoRequest{Sn: sn, DeviceCode: consts.TowerDeviceCode}
	reply, err := pb.SmartAuth.DeviceInfo(context.Background(), req)
	if err != nil {
		logger.Error("func",
			zap.String("call", "pb.SmartAuth.DeviceInfo"),
			zap.String("args", sn),
			zap.String("error", err.Error()))
	}
	return reply, err

}

func GetDeviceInfoById(id int64) (*pb_v1.DeviceInfoByIdReply, error) {
	req := &pb_v1.DeviceInfoByIdRequest{Id:id}
	reply, err := pb.SmartAuth.DeviceInfoById(context.Background(), req)
	if err != nil {
		logger.Error("func",
			zap.String("call", "pb.SmartAuth.DeviceInfoById"),
			zap.String("args", fmt.Sprintf("%d", id)),
			zap.String("error", err.Error()))
	}
	return reply, err

}

func GetDustSns()(*pb_v1.GetAllDustSnReply, error) {
	req := &pb_v1.GetAllDustSnRequest{}
	reply, err := pb.SmartAuth.GetAllDustSn(context.Background(), req)
	if err != nil {
		logger.Error("func",
			zap.String("call", "pb.SmartAuth.GetAllDustSn"),
			zap.String("error", err.Error()))
	}
	return reply, err
}

